use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Anders
Berle
Class
Constructor
Dave
Dieter
Etheridge
Hans
Instance
Jonathan
Kaare
Karen
Meta
Method
Moore
MooseX
Nor
Object
Pearcey
Rasmussen
Ricardo
Rockway
Role
Rolsky
SIGNES
Shawn
Singleton
autarch
berle
code
ether
hdp
irc
jon
kaare
lib
rjbs
